@@ -1,5 +1,5 @@
 package Git::Repository::Command;
-$Git::Repository::Command::VERSION = '1.312';
+$Git::Repository::Command::VERSION = '1.325';
 use strict;
 use warnings;
 use 5.006;
@@ -9,7 +9,6 @@ use Cwd qw( cwd );
 use IO::Handle;
 use Scalar::Util qw( blessed );
 use File::Spec;
-use Config;
 use System::Command;
 
 our @ISA = qw( System::Command );
@@ -24,6 +23,24 @@ for my $attr (qw( cmdline )) {
     *$attr = sub { return @{ $_[0]{$attr} } };
 }
 
+sub _which {
+    my @pathext = ('');
+    push @pathext,
+        $^O eq 'MSWin32' ? split ';', $ENV{PATHEXT}
+      : $^O eq 'cygwin'  ? qw( .com .exe .bat )
+      :                    ();
+
+    for my $path ( File::Spec->path ) {
+        for my $ext (@pathext) {
+            my $binary = File::Spec->catfile( $path, $_[0] . $ext );
+            return $binary if -x $binary && !-d _;
+        }
+    }
+
+    # not found
+    return undef;
+}
+
 # CAN I HAS GIT?
 my %binary;    # cache calls to _is_git
 sub _is_git {
@@ -35,14 +52,18 @@ sub _is_git {
     # and use the arguments as part of the cache key
 
     # compute cache key:
-    # - filename (path):     path
+    # - filename (path-rel): $CWD \0 $PATH
+    # - filename (path):     $PATH
     # - absolute path (abs): empty string
     # - relative path (rel): dirname
     my $path = defined $ENV{PATH} && length( $ENV{PATH} ) ? $ENV{PATH} : '';
-    my ( $type, $key )
-        = ( File::Spec->splitpath($binary) )[2] eq $binary ? ( 'path', $path )
-        : File::Spec->file_name_is_absolute($binary)       ? ( 'abs', '' )
-        :                                                    ( 'rel', cwd() );
+    my ( $type, $key ) =
+        ( File::Spec->splitpath($binary) )[2] eq $binary
+      ? grep( !File::Spec->file_name_is_absolute($_), File::Spec->path )
+          ? ( 'path-rel', join "\0", cwd(), $path )
+          : ( 'path', $path )
+      : File::Spec->file_name_is_absolute($binary) ? ( 'abs', '' )
+      :                                              ( 'rel', cwd() );
 
     # This relatively complex cache key scheme allows PATH or cwd to change
     # during the life of a program using Git::Repository, which is likely
@@ -55,22 +76,11 @@ sub _is_git {
         if exists $binary{$type}{$key}{$binary}{$args};
 
     # compute a list of candidate files (look in PATH if needed)
-    my $git;
-    if ( $type eq 'path' ) {
-        my $path_sep = $Config::Config{path_sep} || ';';
-        my @ext = (
-            '', $^O eq 'MSWin32' ? ( split /\Q$path_sep\E/, $ENV{PATHEXT} ) : ()
-        );
-        ($git) = grep { -x && !-d }
-            map {
-            my $path = $_;
-            map { File::Spec->catfile( $path, $_ ) } map {"$binary$_"} @ext
-            }
-            split /\Q$path_sep\E/, $path;
-    }
-    else {
-        $git = File::Spec->rel2abs($binary);
-    }
+    my $git = $type =~ /^path/
+      ? _which($binary)
+      : File::Spec->rel2abs($binary);
+    $git = File::Spec->rel2abs($git)
+      if defined $git && $type eq 'path-rel';
 
     # if we can't find any, we're done
     return $binary{$type}{$key}{$binary} = undef
@@ -125,6 +135,10 @@ sub new {
             if defined $work_tree;
     }
 
+    # pick up the modified PATH, if any
+    exists $_->{env} and exists $_->{env}{PATH} and $ENV{PATH} = $_->{env}{PATH}
+      for @o;
+
     # extract and process the 'fatal' option
     push @o, {
         fatal => {
@@ -160,20 +174,21 @@ sub final_output {
     my ($self, @cb) = @_;
 
     # get output / errput
-    my ( $stdout, $stderr ) = @{$self}{qw(stdout stderr)};
     my ( @output, @errput );
-    {
-        local $/ = "\n";
-        chomp( @output = <$stdout> );
-        chomp( @errput = <$stderr> );
-    }
+    $self->loop_on(
+        input_record_separator => "\n",
+        stdout => sub { chomp( my $o = shift ); push @output, $o; },
+        stderr => sub { chomp( my $e = shift ); push @errput, $e; },
+    );
 
     # done with it
     $self->close;
 
     # fatal exit codes set by the 'fatal' option
+    # when working with fatal => '!0' it's helpful to be able to show the exit status
+    # so that specific exit codes can be made non-fatal if desired.
     if ( $self->options->{fatal}{ $self->exit } ) {
-        croak join( "\n", @errput ) || 'fatal: unknown git error';
+        croak join( "\n", @errput ) || 'fatal: unknown git error, exit status '.$self->exit;
     }
 
     # something else's wrong
@@ -190,19 +205,13 @@ sub final_output {
 
 1;
 
-# ABSTRACT: Command objects for running git
-
-
 __END__
-=pod
 
 =head1 NAME
 
 Git::Repository::Command - Command objects for running git
 
-=head1 VERSION
-
-version 1.312
+=pod
 
 =head1 SYNOPSIS
 
@@ -234,6 +243,7 @@ version 1.312
     # cut to the chase
     my ( $pid, $in, $out, $err ) = Git::Repository::Command->spawn(@cmd);
 
+
 =head1 DESCRIPTION
 
 L<Git::Repository::Command> is a class that actually launches a B<git>
@@ -275,7 +285,7 @@ should be an array reference with the command and parameters, like this:
 =item C<cwd>
 
 The I<current working directory> in which the git command will be run.
-(C<chdir()> will be called just before lauching the command.)
+(C<chdir()> will be called just before launching the command.)
 
 If not provided, it will default to the root of the Git repository work
 tree (if the repository is bare, then no C<chdir()> will be performed).
@@ -335,6 +345,7 @@ keys in later hashes taking precedence over keys in earlier hashes.
 The L<Git::Repository::Command> object returned by C<new()> has a
 number of attributes defined (see below).
 
+
 =head2 close
 
     $cmd->close();
@@ -362,6 +373,7 @@ warnings. If the git sub-process exited with a status code listed in
 the C<fatal> option, it will C<die()>. The defaults fatal exit codes
 are C<128> (fatal error), and C<129> (usage message).
 
+
 =head2 Accessors
 
 The attributes of a L<Git::Repository::Command> object are also accessible
@@ -419,6 +431,10 @@ The signal, if any, that killed the command.
 
 =back
 
+=head1 AUTHOR
+
+Philippe Bruhat (BooK) <book@cpan.org>
+
 =head1 ACKNOWLEDGEMENTS
 
 The core of L<Git::Repository::Command> has been moved into its own
@@ -437,23 +453,9 @@ Win32 support for L<System::Command> through a quick round of edit
 (on my Linux box) and testing (on his Windows box) during the Perl QA
 Hackathon 2013 in Lancaster.
 
-=head1 BUGS
-
-Please report any bugs or feature requests on the bugtracker website
-http://rt.cpan.org/NoAuth/Bugs.html?Dist=Git-Repository or by email to
-bug-git-repository@rt.cpan.org.
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
-
-=head1 AUTHOR
-
-Philippe Bruhat (BooK) <book@cpan.org>
-
 =head1 COPYRIGHT
 
-Copyright 2010-2014 Philippe Bruhat (BooK), all rights reserved.
+Copyright 2010-2016 Philippe Bruhat (BooK), all rights reserved.
 
 =head1 LICENSE
 
@@ -461,4 +463,3 @@ This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =cut
-
